@@ -1,13 +1,13 @@
 /*
  * Functions for opening and closing files.
  *
- * Copyright 2007 Andrew Wood, distributed under the Artistic License 2.0.
+ * Copyright 2008 Andrew Wood, distributed under the Artistic License 2.0.
  */
 
+#define _GNU_SOURCE 1
+#include <limits.h>
+
 #include <stdio.h>
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #include "options.h"
 
 #include <stdlib.h>
@@ -19,6 +19,9 @@
 #include <unistd.h>
 #include <fcntl.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 /*
  * Try to work out the total size of all data by adding up the sizes of all
