@@ -1,12 +1,9 @@
 /*
  * Signal handling functions.
  *
- * Copyright 2007 Andrew Wood, distributed under the Artistic License 2.0.
+ * Copyright 2008 Andrew Wood, distributed under the Artistic License 2.0.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #include "pv.h"
 
 #include <signal.h>
@@ -18,12 +15,20 @@
 #include <fcntl.h>
 #include <sys/time.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 static int pv__sig_old_stderr;		 /* see pv__sig_ttou() */
 static struct timeval pv__sig_tstp_time; /* see pv__sig_tstp() / __cont() */
 
 struct timeval pv_sig_toffset;		 /* total time spent stopped */
 sig_atomic_t pv_sig_newsize = 0;	 /* whether we need to get term size again */
+sig_atomic_t pv_sig_abort = 0;		 /* whether we need to abort right now */
+
+#ifdef HAVE_IPC
+void pv_crs_needreinit(void);
+#endif
 
 
 /*
@@ -123,6 +128,15 @@ static void pv__sig_winch(int s)
 }
 
 
+/*
+ * Handle termination signals by setting the abort flag.
+ */
+static void pv__sig_term(int s)
+{
+	pv_sig_abort = 1;
+}
+
+
 /*
  * Initialise signal handling.
  */
@@ -181,6 +195,25 @@ void pv_sig_init(void)
 	sigemptyset(&(sa.sa_mask));
 	sa.sa_flags = 0;
 	sigaction(SIGWINCH, &sa, NULL);
+
+	/*
+	 * Handle SIGINT, SIGHUP, SIGTERM by setting a flag to let the
+	 * main loop know it should quit now.
+	 */
+	sa.sa_handler = pv__sig_term;
+	sigemptyset(&(sa.sa_mask));
+	sa.sa_flags = 0;
+	sigaction(SIGINT, &sa, NULL);
+
+	sa.sa_handler = pv__sig_term;
+	sigemptyset(&(sa.sa_mask));
+	sa.sa_flags = 0;
+	sigaction(SIGHUP, &sa, NULL);
+
+	sa.sa_handler = pv__sig_term;
+	sigemptyset(&(sa.sa_mask));
+	sa.sa_flags = 0;
+	sigaction(SIGTERM, &sa, NULL);
 }
 
 
